"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const command_1 = require("@oclif/command");
const ConfigRepository_1 = require("../config/ConfigRepository");
const Resolver_1 = require("../services/Resolver");
const helper_identify_1 = require("@comet-cli/helper-identify");
const Logger_1 = require("../helpers/Logger");
const File_1 = require("../helpers/File");
const fs_extra_1 = require("fs-extra");
class BaseCommand extends command_1.Command {
    constructor(argv, config) {
        super(argv, config);
        this.plugins = [];
        this.warnings = [];
        this.configRepository = new ConfigRepository_1.default();
        this.logger = new Logger_1.default();
    }
    async resolve(file) {
        if (this.signature == null) {
            return;
        }
        const resolver = new Resolver_1.default(this.configRepository, this.signature);
        const source = await fs_extra_1.readFile(file.path(), 'utf8');
        const mediaType = helper_identify_1.identify(source);
        let detected = null;
        switch (mediaType) {
            case 'text/vnd.apiblueprint':
                detected = '@comet-cli/adapter-api-blueprint';
                break;
            case 'application/vnd.oai.openapi':
            case 'application/vnd.oai.openapi+json':
            case 'application/swagger+yaml':
            case 'application/swagger+json':
                detected = '@comet-cli/adapter-openapi';
                break;
            case 'application/raml+yaml':
                detected = '@comet-cli/adapter-raml';
        }
        this.adapter = await resolver.resolveAdapter(detected);
        this.plugins = await resolver.resolvePlugins();
    }
    async loadFile(args) {
        try {
            return new File_1.default(args.input);
        }
        catch (error) {
            this.logger.fail(`${args.input} is not a valid file.\n${error.message}`);
            return this.exit(-1);
        }
    }
    async parseSpec(file) {
        this.logger.spin('Parsing input file');
        let spec;
        try {
            await this.resolve(file);
            const adapter = this.adapter.name();
            spec = await this.adapter.execute(file.path(), this.configRepository.get(`adapters.${adapter}`), this.logger);
        }
        catch (error) {
            this.logger.fail(error.message);
            console.error(error.stack);
            return this.exit(-1);
        }
        this.logger.succeed('Input file parsed');
        return spec;
    }
    async runPlugins(model) {
        try {
            for (let i = 0; i < this.plugins.length; i = i + 1) {
                const plugin = this.plugins[i].name();
                await this.plugins[i].execute(model, this.configRepository.get(`plugins.${plugin}`), this.logger);
            }
        }
        catch (error) {
            this.logger.fail(error.message);
            console.error(error.stack);
            this.exit(-1);
        }
    }
    printWarnings() {
        this.warnings.forEach((warning) => {
            this.logger.warn(warning);
        });
    }
}
exports.default = BaseCommand;
//# sourceMappingURL=BaseCommand.js.map